@@ -1,31 +1,47 @@
 extern crate jsonwebtoken as jwt;
 extern crate rustc_serialize;
 
-use jwt::{encode, decode, Algorithm};
+use jwt::{encode, decode, Header, Algorithm};
 use jwt::errors::{Error};
 
+
 #[derive(Debug, RustcEncodable, RustcDecodable)]
 struct Claims {
     sub: String,
     company: String
 }
 
+// Example validation implementation
+impl Claims {
+    fn is_valid(self) -> bool {
+        if self.company != "ACME".to_owned() {
+            return false;
+        }
+        // expiration etc
+
+        true
+    }
+}
+
 fn main() {
     let my_claims = Claims {
         sub: "b@b.com".to_owned(),
         company: "ACME".to_owned()
     };
     let key = "secret";
-    let token = match encode(&my_claims, key, Algorithm::HS256) {
+    let token = match encode(Header::default(), &my_claims, key.as_ref()) {
         Ok(t) => t,
         Err(_) => panic!() // in practice you would return the error
     };
 
-    let claims = match decode::<Claims>(&token, key, Algorithm::HS256) {
+    let token_data = match decode::<Claims>(&token, key.as_ref(), Algorithm::HS256) {
         Ok(c) => c,
         Err(err) => match err {
             Error::InvalidToken => panic!(), // Example on how to handle a specific error
             _ => panic!()
         }
     };
+    println!("{:?}", token_data.claims);
+    println!("{:?}", token_data.header);
+    println!("{:?}", token_data.claims.is_valid());
 }
