@@ -19,7 +19,7 @@ use crypto::util::fixed_time_eq;
 pub mod errors;
 use errors::Error;
 
-#[derive(Debug, PartialEq, Copy, Clone)]
+#[derive(Debug, PartialEq, Copy, Clone, RustcDecodable, RustcEncodable)]
 /// The algorithms supported for signing/verifying
 pub enum Algorithm {
     HS256,
@@ -51,46 +51,42 @@ impl<T> Part for T where T: Encodable + Decodable {
     }
 }
 
-#[derive(Debug, PartialEq)]
-/// A basic JWT header part, the alg is automatically filled for use
-/// It's missing things like the kid but that's for later
+#[derive(Debug, PartialEq, RustcDecodable, RustcEncodable)]
+/// A basic JWT header part, the alg defaults to HS256 and typ is automatically
+/// set to `JWT`. All the other fields are optional
 pub struct Header {
-    typ: &'static str,
-    alg: Algorithm,
+    typ: String,
+    pub alg: Algorithm,
+    pub jku: Option<String>,
+    pub kid: Option<String>,
+    pub x5u: Option<String>,
+    pub x5t: Option<String>
 }
 
 impl Header {
     pub fn new(algorithm: Algorithm) -> Header {
         Header {
-            typ: "JWT",
+            typ: "JWT".to_owned(),
             alg: algorithm,
+            jku: None,
+            kid: None,
+            x5u: None,
+            x5t: None
         }
     }
 }
 
-impl Part for Header {
-    type Encoded = &'static str;
-
-    fn from_base64<B: AsRef<[u8]>>(encoded: B) -> Result<Self, Error> where Self: Sized {
-        let algoritm = match encoded.as_ref() {
-            b"eyJ0eXAiOiJKV1QiLCJhbGciOiJIUzI1NiJ9" => { Algorithm::HS256 },
-            b"eyJ0eXAiOiJKV1QiLCJhbGciOiJIUzM4NCJ9" => { Algorithm::HS384 },
-            b"eyJ0eXAiOiJKV1QiLCJhbGciOiJIUzUxMiJ9" => { Algorithm::HS512 },
-            _ => return Err(Error::InvalidToken)
-        };
-
-        Ok(Header::new(algoritm))
+impl Default for Header {
+    fn default() -> Header {
+        Header::new(Algorithm::HS256)
     }
+}
 
-    fn to_base64(&self) -> Result<Self::Encoded, Error> {
-        let encoded = match self.alg {
-            Algorithm::HS256 => { "eyJ0eXAiOiJKV1QiLCJhbGciOiJIUzI1NiJ9" },
-            Algorithm::HS384 => { "eyJ0eXAiOiJKV1QiLCJhbGciOiJIUzM4NCJ9" },
-            Algorithm::HS512 => { "eyJ0eXAiOiJKV1QiLCJhbGciOiJIUzUxMiJ9" },
-        };
-
-        Ok(encoded)
-    }
+#[derive(Debug)]
+/// The return type of a successful call to decode(...)
+pub struct TokenData<T: Part> {
+    pub header: Header,
+    pub claims: T
 }
 
 /// Take the payload of a JWT and sign it using the algorithm given.
@@ -114,36 +110,38 @@ fn verify(signature: &str, data: &str, secret: &[u8], algorithm: Algorithm) -> b
     fixed_time_eq(signature.as_ref(), sign(data, secret, algorithm).as_ref())
 }
 
-/// Encode the claims passed and sign the payload using the algorithm and the secret
-pub fn encode<T: Part, B: AsRef<[u8]>>(claims: &T, secret: B, algorithm: Algorithm) -> Result<String, Error> {
-    let encoded_header = try!(Header::new(algorithm).to_base64());
+/// Encode the claims passed and sign the payload using the algorithm from the header and the secret
+pub fn encode<T: Part>(header: Header, claims: &T, secret: &[u8]) -> Result<String, Error> {
+    let encoded_header = try!(header.to_base64());
     let encoded_claims = try!(claims.to_base64());
     // seems to be a tiny bit faster than format!("{}.{}", x, y)
-    let payload = [encoded_header, encoded_claims.as_ref()].join(".");
-    let signature = sign(&*payload, secret.as_ref(), algorithm);
+    let payload = [encoded_header.as_ref(), encoded_claims.as_ref()].join(".");
+    let signature = sign(&*payload, secret.as_ref(), header.alg);
 
     Ok([payload, signature].join("."))
 }
 
+/// Used in decode: takes the result of a rsplit and ensure we only get 2 parts
+/// Errors if we don't
+macro_rules! expect_two {
+    ($iter:expr) => {{
+        let mut i = $iter; // evaluate the expr
+        match (i.next(), i.next(), i.next()) {
+            (Some(first), Some(second), None) => (first, second),
+            _ => return Err(Error::InvalidToken)
+        }
+    }}
+}
+
 /// Decode a token into a Claims struct
 /// If the token or its signature is invalid, it will return an error
-pub fn decode<T: Part>(token: &str, secret: &str, algorithm: Algorithm) -> Result<T, Error> {
-    macro_rules! expect_two {
-        ($iter:expr) => {{
-            let mut i = $iter; // evaluate the expr
-            match (i.next(), i.next(), i.next()) {
-                (Some(first), Some(second), None) => (first, second),
-                _ => return Err(Error::InvalidToken)
-            }
-        }}
-    }
-
+pub fn decode<T: Part>(token: &str, secret: &[u8], algorithm: Algorithm) -> Result<TokenData<T>, Error> {
     let (signature, payload) = expect_two!(token.rsplitn(2, '.'));
 
     let is_valid = verify(
         signature,
         payload,
-        secret.as_bytes(),
+        secret,
         algorithm
     );
 
@@ -157,13 +155,14 @@ pub fn decode<T: Part>(token: &str, secret: &str, algorithm: Algorithm) -> Resul
     if header.alg != algorithm {
         return Err(Error::WrongAlgorithmHeader);
     }
+    let decoded_claims = try!(T::from_base64(claims));
 
-    T::from_base64(claims)
+    Ok(TokenData { header: header, claims: decoded_claims})
 }
 
 #[cfg(test)]
 mod tests {
-    use super::{encode, decode, Algorithm, Header, Part, sign, verify};
+    use super::{encode, decode, Algorithm, Header, sign, verify};
 
     #[derive(Debug, PartialEq, Clone, RustcEncodable, RustcDecodable)]
     struct Claims {
@@ -171,29 +170,6 @@ mod tests {
         company: String
     }
 
-    #[test]
-    fn to_base64() {
-        let expected = "eyJ0eXAiOiJKV1QiLCJhbGciOiJIUzI1NiJ9".to_owned();
-        let result = Header::new(Algorithm::HS256).to_base64();
-
-        assert_eq!(expected, result.unwrap());
-    }
-
-    #[test]
-    fn from_base64() {
-        let encoded = "eyJ0eXAiOiJKV1QiLCJhbGciOiJIUzI1NiJ9";
-        let header = Header::from_base64(encoded).unwrap();
-
-        assert_eq!(header.typ, "JWT");
-        assert_eq!(header.alg, Algorithm::HS256);
-    }
-
-    #[test]
-    fn round_trip_base64() {
-        let header = Header::new(Algorithm::HS256);
-        assert_eq!(Header::from_base64(header.to_base64().unwrap()).unwrap(), header);
-    }
-
     #[test]
     fn sign_hs256() {
         let result = sign("hello world", b"secret", Algorithm::HS256);
@@ -208,22 +184,38 @@ mod tests {
         assert!(valid);
     }
 
+    #[test]
+    fn encode_with_custom_header() {
+        // TODO: test decode value
+        let my_claims = Claims {
+            sub: "b@b.com".to_owned(),
+            company: "ACME".to_owned()
+        };
+        let mut header = Header::default();
+        header.kid = Some("kid".to_owned());
+        let token = encode(header, &my_claims, "secret".as_ref()).unwrap();
+        let token_data = decode::<Claims>(&token, "secret".as_ref(), Algorithm::HS256).unwrap();
+        assert_eq!(my_claims, token_data.claims);
+        assert_eq!("kid", token_data.header.kid.unwrap());
+    }
+
     #[test]
     fn round_trip_claim() {
         let my_claims = Claims {
             sub: "b@b.com".to_owned(),
             company: "ACME".to_owned()
         };
-        let token = encode(&my_claims, "secret", Algorithm::HS256).unwrap();
-        let claims = decode::<Claims>(&token, "secret", Algorithm::HS256).unwrap();
-        assert_eq!(my_claims, claims);
+        let token = encode(Header::default(), &my_claims, "secret".as_ref()).unwrap();
+        let token_data = decode::<Claims>(&token, "secret".as_ref(), Algorithm::HS256).unwrap();
+        assert_eq!(my_claims, token_data.claims);
+        assert!(token_data.header.kid.is_none());
     }
 
     #[test]
     #[should_panic(expected = "InvalidToken")]
     fn decode_token_missing_parts() {
         let token = "eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9";
-        let claims = decode::<Claims>(token, "secret", Algorithm::HS256);
+        let claims = decode::<Claims>(token, "secret".as_ref(), Algorithm::HS256);
         claims.unwrap();
     }
 
@@ -231,7 +223,21 @@ mod tests {
     #[should_panic(expected = "InvalidSignature")]
     fn decode_token_invalid_signature() {
         let token = "eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9.eyJzdWIiOiJiQGIuY29tIiwiY29tcGFueSI6IkFDTUUifQ.wrong";
-        let claims = decode::<Claims>(token, "secret", Algorithm::HS256);
+        let claims = decode::<Claims>(token, "secret".as_ref(), Algorithm::HS256);
         claims.unwrap();
     }
+
+    #[test]
+    fn decode_token_with_bytes_secret() {
+        let token = "eyJ0eXAiOiJKV1QiLCJhbGciOiJIUzI1NiJ9.eyJzdWIiOiIxMjM0NTY3ODkwIiwiY29tcGFueSI6Ikdvb2dvbCJ9.27QxgG96vpX4akKNpD1YdRGHE3_u2X35wR3EHA2eCrs";
+        let claims = decode::<Claims>(token, b"\x01\x02\x03", Algorithm::HS256);
+        assert!(claims.is_ok());
+    }
+
+    #[test]
+    fn decode_token_with_shuffled_header_fields() {
+        let token = "eyJ0eXAiOiJKV1QiLCJhbGciOiJIUzI1NiJ9.eyJjb21wYW55IjoiMTIzNDU2Nzg5MCIsInN1YiI6IkpvaG4gRG9lIn0.SEIZ4Jg46VGhquuwPYDLY5qHF8AkQczF14aXM3a2c28";
+        let claims = decode::<Claims>(token, "secret".as_ref(), Algorithm::HS256);
+        assert!(claims.is_ok());
+    }
 }
